import express from "express";
import CommonUtil from "../../utils/commonUtil.js";
import {Contract, ethers, utils} from "ethers";
import RouterEthAbi from './contract/routerEth.js'
import mathUtil from "../../utils/mathUtil.js";

const router = express.Router();
const ethChainId = 101;
const wrapperEth = "0x72E2F4830b9E45d52F80aC08CB2bEC0FeF72eD9c";
const routerAddress = "0x8731d54E9D02c286767d56ac03e8037C07e01e98";
const routerEthAddress = "0x150f94B44927F078737562f0fcF3C95c01Cc2376";
const network = "https://eth-mainnet.g.alchemy.com/v2/I0grWRjLbj_Rf1WP3QMJ-z3mKp71Hb4L";

const arbChainId = 110;
const wrapperArbEthAddress = "0x82cbecf39bee528b5476fe6d1550af59a9db6fc0";
const routerArbAddress = "0x53bf833a5d6c4dda888f69c22c88c9f356a41614";
const routerArbEthAddress = "0xbf22f0f184bCcbeA268dF387a49fF5238dD23E40";
const arbNetwork = "https://arb-mainnet.g.alchemy.com/v2/8hbABA2L3eaR7VWGY8yk-5p16Aeurf2Z";


router.post("/", async (req, res) => {
    let {wallet, params} = CommonUtil.getParamsAndWallet(req);
    let distChainId;
    let wrap_addr;
    let rt_addr;
    let rt_eth_addr;
    let net_work;
    if (params.distChain === "eth") {
        distChainId = ethChainId;
        wrap_addr = wrapperEth;
        rt_addr = routerAddress;
        rt_eth_addr = routerEthAddress;
        net_work = network;
    } else if (params.distChain === "arb") {
        distChainId = arbChainId;
        wrap_addr = wrapperArbEthAddress;
        rt_addr = routerArbAddress;
        rt_eth_addr = routerArbEthAddress;
        net_work = arbNetwork;
    }
    const provider = ethers.getDefaultProvider(net_work);
    const signer = new ethers.Wallet(wallet.privateKey, provider);
    // const router = new Contract(rt_addr, RouterAbi, signer);
    const routerEth = new Contract(rt_eth_addr, RouterEthAbi, signer);
    let amountNumber = mathUtil.getRandomNumber(params.minAmount, params.maxAmount, params.accuracy);
    amountNumber = utils.parseEther(`${amountNumber}`);
    let crossGas = utils.parseEther(params.crossGas);
    const amountOut = amountNumber.mul(995).div(1000);
    let tx = await routerEth.swapETH(
        distChainId,
        wallet.address,
        wallet.address,
        amountNumber,
        amountOut,
        {
            value: amountNumber.add(crossGas)
        }
    )
    await tx.wait();
    tx = await provider.getTransaction(tx.hash);
    const txReceipt = await provider.getTransactionReceipt(tx.hash);
    const gas = utils.formatEther((txReceipt.gasUsed * tx.gasPrice).toFixed(0));
    const balance = await signer.getBalance()
    const balanceStr = utils.formatEther(balance);
    const value = utils.formatEther(tx.value)
    res.send({
        code: 0,
        msg: "",
        data: {
            txId: tx.hash,
            gas: gas,
            currentBalance: balanceStr,
            status: txReceipt.status,
            value: value
        }
    });
});

export default router;